@@ -15,12 +15,14 @@ __all__ = ['MapTableFile',
            'MTContaminant',
            'MTSediment']
 
-from future.utils import iteritems
+import os
 import logging
+
+from future.utils import iteritems
 import numpy as np
 import pandas as pd
-import os
 from osgeo import gdalconst
+from gazar.grid import resample_grid
 from sqlalchemy import ForeignKey, Column
 from sqlalchemy.types import Integer, Float, String
 from sqlalchemy.orm import relationship
@@ -31,7 +33,6 @@ from ..base.file_base import GsshaPyFileObjectBase
 from .idx import IndexMap
 from ..lib import parsetools as pt
 from ..lib import cmt_chunk as mtc
-from ..lib.grid_tools import resample_grid
 from ..lib.parsetools import valueReadPreprocessor as vrp, valueWritePreprocessor as vwp
 
 log = logging.getLogger(__name__)
@@ -183,7 +184,7 @@ class MapTableFile(DeclarativeBase, GsshaPyFileObjectBase):
                 # Initiate index map write
                 indexMap.write(directory, session=session)
 
-        for mapTable in self.mapTables:
+        for mapTable in self.getOrderedMapTables(session):
             if mapTable.name == 'SEDIMENTS':
                 self._writeSedimentTable(session=session,
                                          fileObject=openFile,
@@ -651,7 +652,6 @@ class MapTableFile(DeclarativeBase, GsshaPyFileObjectBase):
                                dtype={'id':'int', 'description':'str', 'roughness':'float'},
                                )
 
-
         # resample land use grid to gssha grid
         land_use_resampled = resample_grid(land_use_grid,
                                            self.projectFile.getGrid(),
@@ -661,11 +661,11 @@ class MapTableFile(DeclarativeBase, GsshaPyFileObjectBase):
         unique_land_use_ids = np.unique(land_use_resampled.np_array())
 
         #only add ids in index map subset
-        df = df[df['id'].isin(unique_land_use_ids)]
+        df = df[df.id.isin(unique_land_use_ids)]
 
         # make sure all needed land use IDs exist
         for land_use_id in unique_land_use_ids:
-            if land_use_id not in df.index:
+            if land_use_id not in df.id.values:
                 raise IndexError("Land use ID {0} not found in table.".format(land_use_id))
 
         # delete duplicate/old tables with same name if they exist
@@ -687,7 +687,7 @@ class MapTableFile(DeclarativeBase, GsshaPyFileObjectBase):
         mapTable.mapTableFile = self
         # add values to table
         for row in df.itertuples():
-            idx = MTIndex(row.id, row.description, '')
+            idx = MTIndex(str(row.id), row.description, '')
             idx.indexMap = indexMap
             val = MTValue('ROUGH', row.roughness)
             val.index = idx
